# -*- coding:utf-8 -*-


REPL_SET_NAME = 'rs0'

HOSTS = ['server1:27017', 'server2:27017', 'server3:27017']

DB_SETTING = {
    'username': 'root',
    'password': '',
    'host': 'localhost',
    'port': 3306,
    'database': 'blog'
}

DRIVERNAME = 'mysql+pymysql'